
--------------------------------------------------------------------------------
-- Module Declaration
--

if not BigWigs.isWOD then return end -- XXX compat
local mod, CL = BigWigs:NewBoss("Tectus", 994, 1195)
if not mod then return end
mod:RegisterEnableMob(78948)
--mod.engageId = 1722

--------------------------------------------------------------------------------
-- Localization
--

local L = mod:NewLocale("enUS", true)
if L then
	L.earthwarper_trigger1 = "Yjj'rmr" -- Yjj'rmr... Xzzolos...
	L.earthwarper_trigger1 = "Yes, Tectus" -- Yes, Tectus. Bend to... our master's... will....
	L.berserker_trigger1 = "MASTER!" -- MASTER! I COME FOR YOU!
	L.berserker_trigger2 = "Kral'ach" --Kral'ach.... The darkness speaks.... A VOICE!

	L.adds = CL.adds
	L.adds_desc = "Timers for when new adds enter the fight."

	L.tectus = "Tectus"
	L.shard = "Shard"
	L.motes = "Motes"

	L.earthwarper_icon = "spell_shadow_raisedead" -- there's no pale orcish spell icons :(
	L.berserker_icon = "ability_warrior_endlessrage" -- angry orc
end
L = mod:GetLocale()

--------------------------------------------------------------------------------
-- Initialization
--

function mod:GetOptions()
	return {
		{162894, "TANK"}, {162892, "TANK"}, 162968,
		163312,
		{162288, "TANK"}, {162346, "FLASH"}, 162475, "adds", "bosskill",
	}, {
		[162894] = -10061, -- Earthwarper
		[163312] = -10062, -- Berserker
		[162288] = "general",
	}
end

function mod:OnBossEnable()
	self:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT", "CheckBossStatus")

	-- Tectus
	self:Log("SPELL_AURA_APPLIED_DOSE", "Accretion", 162288)
	self:Log("SPELL_AURA_APPLIED", "CrystallineBarrage", 162346)
	self:Log("SPELL_AURA_APPLIED", "CrystallineBarrageDamage", 162370)
	self:Log("SPELL_PERIODIC_DAMAGE", "CrystallineBarrageDamage", 162370)
	self:Log("SPELL_CAST_START", "TectonicUpheaval", 162475)
	self:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "Split", "boss1", "boss2", "boss3")
	-- Earthwarper
	self:Yell("Earthwarper", L.earthwarper_trigger1, L.earthwarper_trigger2)
	self:Log("SPELL_CAST_START", "GiftOfEarth", 162894)
	self:Log("SPELL_AURA_APPLIED", "Petrification", 162892)
	self:Log("SPELL_CAST_START", "EarthenFlechettes", 162968)
	self:Log("SPELL_DAMAGE", "EarthenFlechettesDamage", 162968)
	self:Log("SPELL_MISSED", "EarthenFlechettesDamage", 162968)
	-- Berserker
	self:Yell("Berserker", L.berserker_trigger1, L.berserker_trigger2)
	self:Log("SPELL_CAST_START", "RavingAssault", 163312)

	self:Death("Win", 78948)
end

function mod:OnEngage()
	self:CDBar(162346, 6) -- Crystalline Barrage
	self:CDBar("adds", 10, -10061, L.earthwarper_icon) -- Earthwarper
	self:CDBar("adds", 20, -10062, L.berserker_icon) -- Berserker
end

--------------------------------------------------------------------------------
-- Event Handlers
--

do
	local prev = 0
	function mod:Accretion(args)
		local t = GetTime()
		if self:MobId(args.sourceGUID) ~= 80557 and args.amount > 3 and t-prev > 5 then
			self:Message(args.spellId, "Attention", nil, CL.count:format(args.spellName, args.amount))
		end
	end
end

do
	local prev = 0
	function mod:CrystallineBarrage(args)
		-- stop announcing barrage from Motes to the raid
		local isMe = self:Me(args.destGUID)
		if self:MobId(args.sourceGUID) ~= 80557 or isMe then
			if isMe then
				self:Flash(args.spellId)
			end
			self:TargetMessage(args.spellId, args.destName, "Personal", "Alarm")
		end
		local t = GetTime()
		if t-prev > 10 then -- not sure how out of sync they get
			self:CDBar(args.spellId, 20)
			prev = t
		end
	end
end

do
	local prev = 0
	function mod:CrystallineBarrageDamage(args)
		local t = GetTime()
		if self:Me(args.destGUID) and t-prev > 3 then
			self:Message(args.spellId, "Personal", "Alarm", CL.underyou:format(args.spellName))
			prev = t
		end
	end
end

do
	local prev = 0
	local names = { [78948] = L.tectus, [80551] = L.shard, [80557] = L.motes }

	function mod:TectonicUpheaval(args)
		local t = GetTime()
		local id = self:MobId(args.sourceGUID)
		if id ~= 80557 or t-prev > 5 then -- not Mote or first Mote cast in 5s
			local raidIcon = CombatLog_String_GetIcon(args.sourceRaidFlags)
			self:Message(args.spellId, "Positive", nil, CL.other:format(raidIcon .. names[id], args.spellName))
			if id == 80557 then prev = t end
		end
	end
end

function mod:Split(unit, spellName, _, _, spellId)
	if spellId == 140562 then -- Break Player Targetting (cast when Tectus/Shards die)
		self:StopBar(-10061) -- Earthwarper
		self:StopBar(-10062) -- Berserker
		self:CDBar(162346, 8) -- Crystalline Barrage 7-12s, then every ~20s, 2-5s staggered
	end
end

-- Adds

function mod:Earthwarper(args)
	self:Message("adds", "Attention", "Info", -10061, false)
	self:CDBar("adds", 41, -10061, L.earthwarper_icon)
	self:CDBar(162894, 10) -- Gift of Earth
	self:CDBar(162968, 15) -- Earthen Flechettes
end

function mod:GiftOfEarth(args)
	self:Message(args.spellId, "Urgent", "Alert")
	self:CDBar(args.spellId, 11)
end

function mod:Petrification(args)
	self:TargetMessage(args.spellId, args.destName, "Urgent", "Warning")
end

function mod:EarthenFlechettes(args)
	self:Message(args.spellId, "Attention", self:Tank() and "Alert")
	self:CDBar(args.spellId, 15)
end

do
	local prev = 0
	function mod:EarthenFlechettesDamage(args)
		local t = GetTime()
		if self:Me(args.destGUID) and not self:Tank() and t-prev > 2 then
			self:Message(args.spellId, "Personal", "Alarm", CL.underyou:format(args.spellName))
			prev = t
		end
	end
end

function mod:Berserker(args)
	self:Message("adds", "Attention", "Info", -10062, false)
	self:CDBar("adds", 41, -10062, L.berserker_icon)
	self:CDBar(163312, 13) -- Raving Assault (~10s + 3s cast)
end

function mod:RavingAssault(args)
	self:Message(args.spellId, "Urgent")
end

